@@ -23,6 +23,7 @@ Useful functions used by the rest of paramiko.
 from __future__ import generators
 
 import array
+from binascii import hexlify, unhexlify
 import errno
 import sys
 import struct
@@ -105,14 +106,21 @@ def format_binary_line(data):
     return '%-50s %s' % (left, right)
 
 
+def hexify(s):
+    return hexlify(s).upper()
+
+
+def unhexify(s):
+    return unhexlify(s)
+
+
 def safe_string(s):
-    out = b('')
+    out = ''
     for c in s:
-        i = byte_ord(c)
-        if 32 <= i <= 127:
-            out += byte_chr(i)
+        if (byte_ord(c) >= 32) and (byte_ord(c) <= 127):
+            out += c
         else:
-            out += b('%%%02X' % i)
+            out += '%%%02X' % byte_ord(c)
     return out
 
 
@@ -135,15 +143,14 @@ def tb_strings():
     return ''.join(traceback.format_exception(*sys.exc_info())).split('\n')
 
 
-def generate_key_bytes(hashclass, salt, key, nbytes):
+def generate_key_bytes(hash_alg, salt, key, nbytes):
     """
     Given a password, passphrase, or other human-source key, scramble it
     through a secure hash into some keyworthy bytes.  This specific algorithm
     is used for encrypting/decrypting private key files.
 
-    :param class hashclass:
-        class from `Crypto.Hash` that can be used as a secure hashing function
-        (like ``MD5`` or ``SHA``).
+    :param function hash_alg: A function which creates a new hash object, such
+        as ``hashlib.sha256``.
     :param salt: data to salt the hash with.
     :type salt: byte string
     :param str key: human-entered password or passphrase.
@@ -155,7 +162,7 @@ def generate_key_bytes(hashclass, salt, key, nbytes):
     if len(salt) > 8:
         salt = salt[:8]
     while nbytes > 0:
-        hash_obj = hashclass.new()
+        hash_obj = hash_alg()
         if len(digest) > 0:
             hash_obj.update(digest)
         hash_obj.update(b(key))
